package info.aspirecn.cloud.yysj.controller;

import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RestController;

import info.aspirecn.cloud.yysj.entity.ResponseInfo;
import info.aspirecn.cloud.yysj.entity.request.IdentityInfoRequest;
import info.aspirecn.cloud.yysj.entity.request.LivingInfoRequest;
import info.aspirecn.cloud.yysj.service.EidService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;

/**
 * 身份查询列表、活体查询列表
 *
 * @author baizehui
 */
@Api(tags = "身份查询列表、活体查询列表接口")
@ApiResponses({
        @ApiResponse(code = 200, message = "成功处理请求"),
        @ApiResponse(code = 400, message = "参数错误"),
        @ApiResponse(code = 401, message = "没有权限访问该服务"),
        @ApiResponse(code = 403, message = "权限不足无法访问该服务"),
        @ApiResponse(code = 404, message = "未发现该微服务"),
        @ApiResponse(code = 500, message = "服务器内部错误")
})
@RestController
@Slf4j
public class EidController {
	
	@Autowired
    private EidService eidService;

    @ApiOperation(value = "身份查询校验信息列表接口", notes = "身份查询校验信息列表接口。")
    @PostMapping("/getIdentityList")
    private ResponseInfo getIdentityList(@RequestHeader(required = false) String userId,@RequestBody IdentityInfoRequest  identityInfoRequest, HttpServletResponse response){
        try {
        	if (userId== null&&"".equals(userId)) {
        		return new ResponseInfo().setMessage("userId不能为空");
        	}
        	Map<String, Object> map =eidService.getIdentityList(userId,identityInfoRequest);
        	 Integer total = (Integer)map.get("total");
            response.addHeader("pageNum", Integer.toString(identityInfoRequest.getPageNum()));
            response.addHeader("pageSize", Integer.toString(identityInfoRequest.getPageSize()));
            response.addHeader("pageTotal",   Integer.toString(total%identityInfoRequest.getPageSize() > 0 ? total/identityInfoRequest.getPageSize()+1 : total/identityInfoRequest.getPageSize()));
            response.addHeader("total",Integer.toString(total));
            return new ResponseInfo().setData(map.get("dataList"));
        } catch (Exception e) {
            log.error("身份查询校验信息列表接口,e:{}", e);
        }
		return null;
    }
    
    @ApiOperation(value = "活体查询校验信息列表接口", notes = "活体查询校验信息列表接口。")
    @PostMapping("/getLivingList")
    private ResponseInfo getLivingList(@RequestHeader(required = false) String userId,@RequestBody LivingInfoRequest  livingInfoRequest, HttpServletResponse response){
        try {
        	if (userId== null&&"".equals(userId)) {
        		return new ResponseInfo().setMessage("userId不能为空");
        	}
        	Map<String, Object> map =eidService.getLivingList(userId,livingInfoRequest);
        	 Integer total = (Integer)map.get("total");
            response.addHeader("pageNum", Integer.toString(livingInfoRequest.getPageNum()));
            response.addHeader("pageSize", Integer.toString(livingInfoRequest.getPageSize()));
            response.addHeader("pageTotal",   Integer.toString(total%livingInfoRequest.getPageSize() > 0 ? total/livingInfoRequest.getPageSize()+1 : total/livingInfoRequest.getPageSize()));
            response.addHeader("total",Integer.toString(total));
            return new ResponseInfo().setData(map.get("dataList"));
        } catch (Exception e) {
            log.error("活体查询校验信息列表接口,e:{}", e);
        }
		return null;
    }
   
}
